local dialplanapi = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------

local function __sendRequest(obj, url, method, data)
    local hc = require"core.httpClient"
    local r = hc:new()
    local request = string.format("%s://%s:%s/api/%s/PBX/Dialplans/%s",
                                    obj.APITransport, obj.APIHost, obj.APIPort, obj.APIVer, url)

    local options = {authUser = obj.APIUser,
                     authPass = obj.APIPass}

    if data then
        options["headers"] = {["Content-Type"] = "application/json"}
        options["content"] = data
    end
    
    local response, err = r:request(method, request, options)
    if err then
        return false, err
    end
    if response.code ~= 200 then
        return false, string.format("return code: %s", response.code or "unknown")
    end
    if not response.body or type(response.body) ~= "string" then
        return false, "unknown http error(1)"
    end

    local rv = require"dkjson".decode(response.body)
    if not rv then
        return false, "json decode error"
    end

    if rv.type == "result" then
        return rv.result
    elseif rv.type == "error" then
        return false, rv.reason or "unknown http error(2)"
    else
        return false, "unknown http error(3)"
    end
end


local function __getDialplanId(obj, dpname)
    local dialplans, err = obj:getDialplansList(obj)
    if err then
        return false, err
    end

    for _, dialplan in ipairs(dialplans or {}) do
        if dialplan["name"] == dpname then
            return dialplan["id"]
        end
    end
    return 0
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local api = {     -- meta class
     APIHost      = "127.0.0.1"
    ,APIPort      = "80"
    ,APITransport = "http"
    ,APIVer       = "v1"
}


function api:getDialplansList()
    local rv, err = __sendRequest(self, "", "GET")
    if err then
        return false, err
    end
    return rv.records
end


function api:getDialplan(dpid)
    if not dpid or dpid == "" then
        return false, "empty dialplan id"
    end

    return __sendRequest(self, string.format("%s/", dpid), "GET")
end


function api:deleteDialplan(dpid)
    if not dpid or dpid == "" then
        return false, "empty dialplan id"
    end

    return __sendRequest(self, string.format("%s/", dpid), "DELETE")
end


function api:addDialplan(obj)
    if type(obj) ~= "table" then
        return false, "dialplan object is not a table"
    end

    if not obj.name then
        return false, "dialplan name is required"
    end

    local json = require"dkjson".encode(obj)
    if not json or json == "null" then
        return false, "json encode error"
    end
    return __sendRequest(self, "", "POST", json)
end


function api:deleteDialplans(dpid)
    return false, "not implemented"
end


function api:updateDialplan(dpid, obj)
    if not dpid or dpid == "" then
        return false, "empty dialplan id"
    end

    if type(obj) ~= "table" then
        return false, "dialplan object is not a table"
    end

    obj.name = nil

    local json = require"dkjson".encode(obj)
    if not json or json == "null" then
        return false, "json encode error"
    end
    return __sendRequest(self, string.format("%s/", dpid), "PUT", json)
end


function api:getDialplanByName(dpname)
    if not dpname or dpname == "" then
        return false, "empty dialplan name"
    end

    local dpid, err = __getDialplanId(self, dpname)
    if err then
        return false, err
    end
    if dpid == 0 then
        return false, "context not found"
    end

    return self:getDialplan(dpid)
end


function api:deleteDialplanByName(dpname)
    if not dpname or dpname == "" then
        return false, "empty dialplan name"
    end

    local dpid, err = __getDialplanId(self, dpname)
    if err then
        return false, err
    end
    if dpid == 0 then
        return false, "context not found"
    end

    return self:deleteDialplan(dpid)
end


function api:addOrUpdateDialplanByName(obj)
    if type(obj) ~= "table" then
        return false, "dialplan object is not a table"
    end

    local dpname = obj.name
    local dpid, err = __getDialplanId(self, dpname)
    if err then
        return false, err
    end

    if dpid == 0 then
        return self:addDialplan(obj)
    else
        return self:updateDialplan(dpid, obj)
    end
end


-- Class constructor
function dialplanapi:new(user, pass, host, port, transp, apiver)
    if not user or not pass then
        return false, "no credentials"
    end

    local obj = {}
    setmetatable(obj, {__index = api})
    
    -- initialize object data
    obj.APIUser = user
    obj.APIPass = pass

    if host then
        obj.APIHost = host
    end

    if port then
        obj.APIPort = port
    end

    if transp then
        obj.APITransport = transp
    end

    if apiver then
        obj.APIVer = apiver 
    end

    return obj
end

return dialplanapi
