#!/usr/bin/lua

package.path = package.path .. ";/var/lib/callweaver/dialplan/failover/configuration_merger/?.lua"

local conf = require"config"
local notify = require"libs.notify"

LOG = require"libs.log"
LOG.outfile = conf.logfile

local function write_pid()
	local pf = io.open(conf.pid_file, "w")
	pf:write(require"posix.unistd".getpid())
	pf:close()
end

local function lock()
	local pf = io.open(conf.pid_file)
	if pf then
		local pid = pf:read("*a")
		pf:close()

		local runcmd = require"posix".basename(arg[0])
		local plist = require"core.pbxutils".run("ps -e")
		local found = false
		for line in string.gmatch(plist, "(.-)\n") do
			if string.find(line, pid) and string.find(line, runcmd) then
				found = true
				break
			end
		end
		if found then
			LOG.warn("sync is already running")
			os.exit(2)
		else
			LOG.warn("previous sync terminated, remove lock")
			write_pid()
		end
	else
		write_pid()
	end
end

local function unlock()
	os.remove(conf.pid_file)
end

lock()

-- run with one id support
local filter = nil
if arg[1] == "-pbx" then
	if arg[2] then
		filter = {arg[2]}
	else
		LOG.error("no pbx name defined")
		os.exit(1)
	end
end

LOG.info("Sync started")

LOG.debug("load entrypoints ...")
local entrypoints = require"entrypoints"
local steps = entrypoints.steps

LOG.debug("init inventory ...")
local inventory = entrypoints.inventory
local res, err = inventory.init(filter)
if not res then
	LOG.error("inventory error:", err)
	os.exit(1)
end

notify.data["start"] = os.time()
local pbxses = inventory.get_pbxses()
LOG.info(#pbxses, "pbxses found in inventory")
LOG.debug("pbxses in inventory:", table.concat(pbxses, ","))

-- ansible required as base component
local function sync_password_with_ansible()
       local file = "/var/lib/callweaver/dialplan/failover/configuration_merger/playbooks/extra.json"
       local extra = nil
       local fp = io.open(file)
       if fp then
               extra = require"dkjson".decode(fp:read("*a"))
               io.close (fp)
       end
       extra["ansible_password"] = conf["dstpbx"]["apipass"]
       fp = io.open(file, "w")
       fp:write(require"dkjson".encode(extra))
       fp:close()
end
sync_password_with_ansible()

-- override evil iptable ssh rules
local function allow_sync()
    local utils = require"core.pbxutils"
    local playbook = "/var/lib/callweaver/dialplan/failover/configuration_merger/playbooks/allow.yml"
    local extra = "/var/lib/callweaver/dialplan/failover/configuration_merger/playbooks/extra.json"
    local remote_host = conf["dstpbx"]["apihost"]
    local sync_host = conf["srcpbx"]["apihost"]

    local command = 'ansible-playbook -i %s, %s --extra-vars "@%s" --extra-vars "workhorse=%s"'
    local out = utils.run(command, remote_host, playbook, extra, sync_host)
end
allow_sync()

local count = 0
for _, pbx in pairs(pbxses) do
	LOG.info("start processing of", pbx)
	local meta = inventory.get_pbx_meta(pbx)
	local task = require"core.pbxutils".tcopy(meta)
	task.config = conf
	task.modules = {}
	local ok = nil

	local function __cleanup()
		LOG.debug("run cleanup on", pbx)
		for _, step in pairs(steps) do
			ok, err = pcall(entrypoints[step]["cleanup"], pbx, task)
			if not ok then
				LOG.warn(pbx, "cleanup error:", err)
			end
		end
		inventory.cleanup(pbx)
	end

	inventory.cleanup(pbx) -- remove tmp files in case of previous crash
	local pbxerr = false
	res, err = inventory.prepare(pbx)
	if not res then
		LOG.warn(pbx, "skipped, inventory prepare error:", err)
		pbxerr = true
		inventory.cleanup(pbx)
	end

	if not pbxerr then
		LOG.info("starting prepare steps on", pbx)
		for _, step in pairs(steps) do
			LOG.debug("prepare with step", step)
			ok, res, err = pcall(entrypoints[step]["prepare"], pbx, task)
			if not ok or not res then
				LOG.warn(pbx, "skipped, prepare error:", ok and err or res)
				pbxerr = true
				__cleanup()
				break
			end
		end
	end

	if not pbxerr then
		LOG.info("starting enroll steps on", pbx)
		for _, step in pairs(steps) do
			LOG.debug("enroll with step", step)
			ok, res, err = pcall(entrypoints[step]["enroll"], pbx, task)
			if not ok or not res then
				LOG.warn(pbx, "skipped, enroll error:", ok and err or res)
				pbxerr = true
				__cleanup()
				break
			end
		end
	end

	if not pbxerr then
		__cleanup()
		count = count + 1
	else
		notify.data["fails"][#(notify.data["fails"]) + 1] = pbx
	end

end

notify.data["count"] = count + #notify.data["fails"]
notify.data["stop"] = os.time()
notify.send_email()

LOG.info(count, "pbxses processed with sucess, end sync")
unlock()
