local api_sync = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------

local function __fix_sound_path(path, host)
    if not path or path == "" then
        return ""
    end

    if not host or host == "" then
        return path
    end

    local default_sounds_file = "/var/lib/wms/exclude_custom_sounds.txt"
    local sound_files = {}
    if require"core.fsutils".is_file(default_sounds_file) then
        local fp = io.open(default_sounds_file)
        if fp then
            local content = fp:read("*a")
            io.close (fp)
            if content then
                for file in string.gmatch(content, "[^\r\n]+") do
                    table.insert(sound_files, require"core.pbxutils".split(file, "%.")[1])
                end
            end
        end
    end

    local isCustomSound = true
    for _, cf in pairs(sound_files) do
        if string.find(path, cf, 1, true) then
            isCustomSound = false
            break
        end
    end

    if isCustomSound then
        path = string.format("%s/%s", host, path)
    end
    return path
end


local fix_app = {
     Goto = function(params, options)
        if params.dialplan == options.context then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,GotoIf = function(params, options)
        if params.dialplan == options.context then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,Hangup = function(params, options)
        return params
    end
    ,Playback = function(params, options)
        params.filename = __fix_sound_path(params.filename, options.host)
        return params
    end
    ,Set = function(params, options)
        return params
    end
    ,Queue = function(params, options)
        if params.queueId and params.queueId ~= "" then
            params.queueId = options.prefix + tonumber(params.queueId)
        end
        if params.msgForOperator and params.msgForOperator ~= "" then
            params.msgForOperator = __fix_sound_path(params.msgForOperator, options.host)
        end
        return params
    end
    ,CustomApp = function(params, options)
        if string.find(params.cwApp, "QUEUE_MEMBER") then
            params.cwApp = "Noop(Function QUEUE_MEMBER is not supported on failover)"
        end
        return params
    end
    ,Timetable = function(params, options)
        if params.timetableId and params.timetableId ~= "" then
            params.timetableId = options.prefix + tonumber(params.timetableId)
        end
        if params.dialplan == options.context then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,DialPhone = function(params, options)
        params.msgForOperator = __fix_sound_path(params.msgForOperator, options.host)
        return params
    end
    ,GotoIfGroupCount = function(params, options)
        params.groupname = ""
        if params.dialplan == options.context then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,VoiceMail = function(params, options)
        return params
    end
    ,VerifyCaller = function(params, options)
        if params.dialplan == options.context then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,Answer = function(params, options)
        return params
    end
    ,Background = function(params, options)
        params.filename = __fix_sound_path(params.filename, options.host)
        return params
    end
    ,GotoIfSwitchIsOn = function(params, options)
        if params.switchId and params.switchId ~= "" then
            params.switchId = options.prefix + tonumber(params.switchId)
        end
        if params.dialplan == options.context then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,WaitExten = function(params, options)
        return params
    end
    ,GotoIf3SwitchIsOnOrBlinking = function(params, options)
        if params.switchId and params.switchId ~= "" then
            params.switchId = options.prefix + tonumber(params.switchId)
        end
        if params.dialplanOn == options.context then
            params.dialplanOn = string.format("%s_%s", options.host, params.dialplanOn)
        end
        if params.dialplanBlinking == options.context then
            params.dialplanBlinking = string.format("%s_%s", options.host, params.dialplanBlinking)
        end
        return params
    end
    ,DialRemotePbx = function(params, options)
        return params
    end
}


----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
function api_sync.prepare(host, task)
    if not host or host == "" then
        return false, "empty host"
    end

    if not (task.config and task.config.srcpbx) then
        return false, "configuration -> dsthost is missing"
    end

    if not task.config.srcpbx.apiuser then
        return false, "api user is not specified"
    end

    if not task.config.srcpbx.apipass then
        return false, "api password is not specified"
    end

    if not task.config.srcpbx.mergedcontext then
        return false, "context for merging is not specified"
    end

    local prefix, err = require"libs.idrange".get_range(host)
    if not prefix then
        return false, err
    end

    local api, err = require"libs.dialplanapi":new(task.config.srcpbx.apiuser, task.config.srcpbx.apipass, task.config.srcpbx.apihost,
                                                    task.config.srcpbx.apiport, task.config.srcpbx.apiproto, task.config.srcpbx.apiverison)
    if err then
        return false, err
    end

    local data, err = api:getDialplanByName(task.config.srcpbx.mergedcontext)
    if err then
        return false, err
    end

    local rv = {}
    rv.name = string.format("%s_%s", host, data.name)
    rv.description = data.description
    rv.numbers = {}
    rv.includedDialplans = {}

    local options = {host = host, prefix = prefix, context = task.config.srcpbx.mergedcontext}

    for _, rec in ipairs(data.numbers) do
        local apps = {}
        for _, app in ipairs(rec.apps) do
            if fix_app[app.name] then
                table.insert(apps, {name = app.name, params = fix_app[app.name](app.params, options)})
            elseif not task.config.srcpbx.removeUnsupportedApps then
                table.insert(apps,  {name = app.name, params = app.params})
            end
        end
        if #apps ~= 0 then
            table.insert(rv.numbers, {number = rec.number, apps = apps})
        end
    end

    task.modules.api_sync = {}
    task.modules.api_sync.data = rv
    return true
end


function api_sync.enroll(host, task)
    if not (task.config and task.config.dstpbx) then
        return false, "configuration -> dsthost is missing"
    end

    if not task.config.dstpbx.apiuser then
        return false, "api user is not specified"
    end

    if not task.config.dstpbx.apipass then
        return false, "api password is not specified"
    end

    if not (task.modules.api_sync and task.modules.api_sync.data) then
        return false, "data from prepare function not found"
    end

    local api, err = require"libs.dialplanapi":new(task.config.dstpbx.apiuser, task.config.dstpbx.apipass, task.config.dstpbx.apihost,
                                                   task.config.dstpbx.apiport, task.config.dstpbx.apiproto, task.config.dstpbx.apiverison)
    if err then
        return false, err
    end

    return api:addOrUpdateDialplanByName(task.modules.api_sync.data)
end


function api_sync.cleanup(host, task)
    return true
end


return api_sync
